#ifndef __UCHAR_H__
#define __UCHAR_H__

#include <d3dx9.h>
#include <string>
#include <vector>
#include "UMap.h"

class FrameEx : public D3DXFRAME
{
    D3DXMATRIX combinedTransform;
public :
    D3DXMATRIX &getCombindedTransform()
    {
        return combinedTransform;
    }
};

class AllocMeshHierarchy : public ID3DXAllocateHierarchy
{
public:
    HRESULT STDMETHODCALLTYPE CreateFrame(
        THIS_ LPCSTR Name,
        LPD3DXFRAME *ppNewFrame );

    HRESULT STDMETHODCALLTYPE CreateMeshContainer(
        THIS_ LPCSTR Name,
        const D3DXMESHDATA *pMeshData,
        const D3DXMATERIAL *pMaterials,
        const D3DXEFFECTINSTANCE *pEffectInstances,
        DWORD NumMaterials,
        const DWORD *pAdjacency,
        LPD3DXSKININFO pSkinInfo,
        LPD3DXMESHCONTAINER *ppNewMeshContainer);

    HRESULT STDMETHODCALLTYPE DestroyFrame(
        THIS_ LPD3DXFRAME pFrameToFree );

    HRESULT STDMETHODCALLTYPE DestroyMeshContainer(
        THIS_ LPD3DXMESHCONTAINER pMeshContinerToFree );

};

enum ANIMATIONSET_INDEX
{
	WAVE_INDEX   = 0,
	JOG_INDEX    = 1,
	WALK_INDEX   = 2,
	LOITER_INDEX = 3
};

class UChar
{
    // position
    D3DXVECTOR3 m_vPos;
    // ̵ϴ  ǥ
    D3DXVECTOR3 m_vDst;
    //  , x-z 鿡 x࿡ 
    float m_fYaw;
    // ĳ  
    D3DXVECTOR3 m_vDir;
    // ĳ ӵ
    float m_fSpeed;

    ANIMATIONSET_INDEX m_aniIdx;

    // scale
    static float m_scale;

    // a pointer to a map
    UMap * m_map;

    // direct3d device
    LPDIRECT3DDEVICE9 m_pd3dDevice;

    D3DXFRAME* m_root;

    ID3DXAnimationController* m_animCtrl;

    ID3DXSkinInfo* m_skinInfo;

    DWORD m_numBones;

    std::vector<D3DXMATRIX> m_finalTransforms;
    std::vector<D3DXMATRIX*> m_combinedTransforms;

    DWORD m_maxVertInfluences;
    
    ID3DXMesh* m_skinnedMesh;

    ID3DXEffect* m_effect;

    D3DXHANDLE m_hTech;
    D3DXHANDLE m_hWorldViewProj;
    D3DXHANDLE m_hFinalTransforms;
    D3DXHANDLE m_hTex;

public :
    // constructor
    UChar();

    // initialize device objects
    HRESULT initDeviceObjects( LPDIRECT3DDEVICE9 pd3dDevice, const std::string &charFileName, UMap* pmap);
    
    D3DXFRAME* findNodeWithMesh( D3DXFRAME* frame );
    HRESULT buildSkinnedMesh( ID3DXMesh* mesh );
    void buildCombinedTransforms();
    HRESULT buildEffect();
    HRESULT buildMaterials( D3DXMESHCONTAINER* meshContainer );

    // release sources
    void release();

    void frameMove( float deltaTime, D3DXMATRIX& worldViewProj );
    void combineTransforms( FrameEx* frame, D3DXMATRIX &P );

    void play( ANIMATIONSET_INDEX idx );

    // make this character move to the destination
    void move( D3DXVECTOR3 &dst );

    // render a scene
    void render();

    // get a position of a char
    inline D3DXVECTOR3 getPosition()
    {
        return m_vPos;
    }

    static const int MAX_NUM_BONES_SUPPORTED = 35;
};

#endif